/*
	bindings.cpp

	Implementation for key and variable bindings.

	Release: $Name$
	Version: $Revision$
	Last Mod Date: $Date$

	This program is free software; you can redistribute it and/or modify
	it under the terms of the GNU General Public License as published by
	the Free Software Foundation; either version 2 of the License, or
	(at your option) any later version.

	This program is distributed in the hope that it will be useful,
	but WITHOUT ANY WARRANTY; without even the implied warranty of
	MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
	GNU General Public License for more details.

	You should have received a copy of the GNU General Public License
	along with this program; if not, write to the Free Software
	Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "bindings.h"

void CBindings::BindFloat( const string& strName, float* pfValue )
{
	_FloatBindings[ strName ] = pfValue;
}

void CBindings::BindDouble( const string& strName, double* pdValue )
{
	_DoubleBindings[ strName ] = pdValue;
}

void CBindings::BindLong( const string& strName, long* plValue)
{
	_LongBindings[ strName ] = plValue;
}

void CBindings::BindString( const string& strName, string* pstrValue )
{
	_StringBindings[ strName ] = pstrValue;
}

void CBindings::BindEventVariable( const string& strName, bool* pbEvent )
{
	_EventVariableBindings[ strName ] = pbEvent;
}

/*void CBindings::BindEventFunction( const string& strName, PFN_EVENT_FUNCTION pEventFunction )
{
	_EventFunctionBindings[ strName ] = pEventFunction;
}

void CBindings::BindEventParameterFunction( const string& strName, PFN_EVENT_PARAMETER_FUNCTION pParameterFunction )
{
	_ParameterFunctionBindings[ strName ] = pParameterFunction;
}

void CBindings::BindOnOffFunction( const string& strName, PFN_ON_OFF_FUNCTION pOn, PFN_ON_OFF_FUNCTION pOff )
{
	_OnOffFunctionBindings[ strName ] = pOn;
}*/

void CBindings::BindAxisVariable( const string& strName, float* pfAxisMovement )
{
	_AxisBindings[ strName ] = pfAxisMovement;
}

void CBindings::PutFloat( const string& strName, float fValue )
{
	FloatBindings::iterator it = _FloatBindings.find( strName );
	if ( it == _FloatBindings.end() )
		return;

	*(*it).second = fValue;
}

void CBindings::PutDouble( const string& strName, double dValue )
{
	DoubleBindings::iterator it = _DoubleBindings.find( strName );
	if ( it == _DoubleBindings.end() )
		return;

	*(*it).second = dValue;
}

void CBindings::PutLong( const string& strName, long lValue )
{
	LongBindings::iterator it = _LongBindings.find( strName );
	if ( it == _LongBindings.end() )
		return;

	*(*it).second = lValue;
}

void CBindings::PutString( const string& strName, string strValue )
{
	StringBindings::iterator it = _StringBindings.find( strName );
	if ( it == _StringBindings.end() )
		return;

	*(*it).second = strValue;
}

float CBindings::GetFloat( const string& strName )
{
	FloatBindings::iterator it = _FloatBindings.find( strName );
	if ( it == _FloatBindings.end() )
		return 0;

	return *(*it).second;
}

double CBindings::GetDouble( const string& strName )
{
	DoubleBindings::iterator it = _DoubleBindings.find( strName );
	if ( it == _DoubleBindings.end() )
		return 0;

	return *(*it).second;
}

long CBindings::GetLong( const string& strName )
{
	LongBindings::iterator it = _LongBindings.find( strName );
	if ( it == _LongBindings.end() )
		return 0;

	return *(*it).second;
}

string CBindings::GetString( const string& strName )
{
	StringBindings::iterator it = _StringBindings.find( strName );
	if ( it == _StringBindings.end() )
		return "";

	return *(*it).second;
}

/*CBindings::PFN_EVENT_FUNCTION CBindings::GetEventFunction( const string& strName )
{
	EventFunctionBindings::iterator it = _EventFunctionBindings.find( strName );
	if ( it == _EventFunctionBindings.end() )
		return NULL;

	return *(*it).second;
}

CBindings::PFN_EVENT_PARAMETER_FUNCTION CBindings::GetParameterFunction( const string& strName )
{
	ParameterFunctionBindings::iterator it = _ParameterFunctionBindings.find( strName );
	if ( it == _ParameterFunctionBindings.end() )
		return NULL;

	return *(*it).second;
}

CBindings::PFN_ON_OFF_FUNCTION CBindings::GetOnOffFunction( const string& strName )
{
	OnOffFunctionBindings::iterator it = _OnOffFunctionBindings.find( strName );
	if ( it == _OnOffFunctionBindings.end() )
		return NULL;

	return *(*it).second;
}*/

bool* CBindings::GetEventVariable( const string& strName )
{
	EventVariableBindings::iterator it = _EventVariableBindings.find( strName );
	if ( it == _EventVariableBindings.end() )
		return NULL;

	return (*it).second;
}

float* CBindings::GetAxisVariable( const string& strName )
{
	AxisBindings::iterator it = _AxisBindings.find( strName );
	if ( it == _AxisBindings.end() )
		return NULL;

	return (*it).second;
}

