/***************************************************************************
                          bindings.h  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/


#ifndef BINDINGS_H
#define BINDINGS_H

#include "global.h"

#include <string>
#include <map>

using namespace std;

class CBindings
{
public:
	typedef void (*PFN_EVENT_FUNCTION)();
	typedef void (*PFN_EVENT_PARAMETER_FUNCTION)(string strParameter);
	typedef void (*PFN_ON_OFF_FUNCTION)();

	void BindFloat( const string& strName, float* pfValue );
	void BindDouble( const string& strName, double* pdValue );
	void BindLong( const string& strName, long* plValue );
	void BindString( const string& strName, string* pstrValue );
	void BindEventVariable( const string& strName, bool* pbEvent );
	void BindAxisVariable( const string& strName, float* pfAxisMovement );

	/*
	void BindOnOffFunction( const string& strName, PFN_ON_OFF_FUNCTION pOn, PFN_ON_OFF_FUNCTION pOff );
	void BindEventFunction( const string& strName, PFN_EVENT_FUNCTION pEventFunction );
	void BindEventParameterFunction( const string& strName, PFN_EVENT_PARAMETER_FUNCTION pParameterFunction );
*/
	void PutFloat( const string& strName, float fValue );
	void PutDouble( const string& strName, double dValue );
	void PutLong( const string& strName, long lValue );
	void PutString( const string& strName, string strValue );

	float GetFloat( const string& strName );
	double GetDouble( const string& strName );
	long GetLong( const string& strName );
	string GetString( const string& strName );
	bool* GetEventVariable( const string& strName );
	float* GetAxisVariable( const string& strName );
/*	PFN_EVENT_FUNCTION GetEventFunction( const string& strName );
	PFN_EVENT_PARAMETER_FUNCTION GetParameterFunction( const string& strName );
	PFN_ON_OFF_FUNCTION GetOnOffFunction( const string& strName );*/

protected:
	typedef map< string, float* > FloatBindings;
	typedef map< string, double* > DoubleBindings;
	typedef map< string, long* > LongBindings;
	typedef map< string, string* > StringBindings;
	typedef map< string, PFN_EVENT_FUNCTION > EventFunctionBindings;
	typedef map< string, PFN_EVENT_PARAMETER_FUNCTION > ParameterFunctionBindings;
	typedef map< string, bool* > EventVariableBindings;
	typedef map< string, PFN_ON_OFF_FUNCTION > OnOffFunctionBindings;
	typedef map< string, float* > AxisBindings;

	FloatBindings _FloatBindings;
	DoubleBindings _DoubleBindings;
	LongBindings _LongBindings;
	StringBindings _StringBindings;
	EventFunctionBindings _EventFunctionBindings;
	ParameterFunctionBindings _ParameterFunctionBindings;
	OnOffFunctionBindings _OnOffFunctionBindings;
	EventVariableBindings _EventVariableBindings;
	AxisBindings _AxisBindings;
};

#endif

