/***************************************************************************
clouds.cpp  -  description
-------------------
begin                : Sun Oct 1 2000
copyright            : (c) 2000 by Matthew Mastracci
email                : mmastrac@canada.com
***************************************************************************/


#include "clouds.h"
#include "glplatform.h"
#include <math.h>
#include <stdlib.h>

#define PI	3.14159265323
#define DEGTORAD( x ) float( float(x) * PI / 180 )
#define RADTODEG( x ) float( float(x) * 180 / PI )

CClouds::CClouds()
{
	_nSkyDisplayList = -1;
	_lOffset[0] = 0;
	_lOffset[1] = 0;
	_lOffset[2] = 0;
	_lOffset[3] = 0;

	g_lCloudDetailLevel = 1;
	g_lCloudTextureSegments = 5;
	g_fCloudSharpness = 0.95;
	g_lCloudCover = 220;
}

//===========================================================================================================================

CClouds::~CClouds()
{
}

//===========================================================================================================================

void CClouds::_FillRandomTexture( unsigned char bTexture[32][32] )
{
	unsigned char bRandomTexture[32][32];
	
	int x, y;
	
	for ( x = 0; x < 32; x++ )
	{
		for ( y = 0; y < 32; y++ )
		{
			unsigned char c = rand() % 256;
			bRandomTexture[x][y] = c;
		}
	}
	
	for ( x = 0; x < 32; x++ )
	{
		for ( y = 0; y < 32; y++ )
		{
			long x1, x2, y1, y2;
			
			if ( x == 0 )
				x1 = 31;
			else
				x1 = x - 1;
			
			x2 = ( x + 1 ) % 32;
			
			if ( y == 0 )
				y1 = 31;
			else
				y1 = y - 1;
			
			y2 = ( y + 1 ) % 32;

			long lTotal =
				bRandomTexture[x1][y1] + bRandomTexture[x1][y2] + bRandomTexture[x2][y1] + bRandomTexture[x2][y2] +
				bRandomTexture[x1][y] * 2 + bRandomTexture[x2][y] * 2 + bRandomTexture[x][y1] * 2 + bRandomTexture[x][y2] * 2 +
				bRandomTexture[x][y] * 4;
			
			lTotal = lTotal / 16;
			
			bTexture[x][y] = (unsigned char)lTotal;
		}
	}
}

//===========================================================================================================================

void CClouds::_CreateRandomTextures()
{
	for ( int i = 0; i < _lCloudTextures; i++ )
	{
		long lTiledImageSize = _lCloudTextureDimension >> i;
		long lTileCount = lTiledImageSize >> 5;
		
		_FillRandomTexture( _Textures[ i ] );
		
		unsigned char* pImage = _LargeTextureScratchpad;
		
		// For each time we need to tile the image in the Y direction
		for ( long lTileY = 0; lTileY < lTileCount; lTileY++ )
		{
			// For each row in the image
			for ( long lRow = 0; lRow < 32; lRow++ )
			{
				for ( long lTileX = 0; lTileX < lTileCount; lTileX++ )
				{
					memcpy( pImage, &_Textures[ i ][ lRow ][ 0 ], 32 );
					pImage += 32;
				}
			}
		}
		
		gluScaleImage(
			GL_LUMINANCE,
			lTiledImageSize, lTiledImageSize, GL_UNSIGNED_BYTE, _LargeTextureScratchpad,
			_lCloudTextureDimension, _lCloudTextureDimension, GL_UNSIGNED_BYTE, _LargeTextures + _lCloudTextureSize * i
		);
	}

	for ( long lDensity = 0; lDensity < 255; lDensity++ )
		_CloudLookupTable[ lDensity ] = (unsigned char)( 255.0f - ( 255.0f * pow( g_fCloudSharpness, (float)lDensity ) ) );
}

//===========================================================================================================================

void CClouds::GLSetup()
{
	float fDetail = log( g_lCloudDetailLevel ) / log( 2 ) + 4;

	// Do a quick (not perfect) rounding hack to get the detail of lower levels up a bit
	_lCloudTextures = (long)( fDetail * 10 );
	if ( _lCloudTextures % 10 >= 5 )
		_lCloudTextures = _lCloudTextures / 10 + 1;
	else
		_lCloudTextures = _lCloudTextures / 10;

	_lCloudTextureDimension = 256 * g_lCloudDetailLevel;
	_lCloudTextureSize = _lCloudTextureDimension * _lCloudTextureDimension;
	_lCloudFinalTextures = g_lCloudDetailLevel * g_lCloudDetailLevel;

	_Textures = new SmallTexture[ _lCloudTextures ];
	_LargeTextures = new unsigned char[ _lCloudTextures * _lCloudTextureSize ];
	_LargeTextureScratchpad = new unsigned char[ _lCloudTextureSize ];
	_FinalTextures = new FinalTexture[ _lCloudFinalTextures ];

	_CreateRandomTextures();

	_lOffset[0] += 1;
	_lOffset[1] += 1;
	_lOffset[2] += 1;
	_lOffset[3] += 1;

	for ( int i = 0; i < _lCloudFinalTextures; i++ )
	{
		// Identify which square of the giant texture we are working on
		long lX = ( i % g_lCloudDetailLevel ) * 256;
		long lY = ( i / g_lCloudDetailLevel ) * 256;

  		for ( int x = 0; x < 256; x++ )
  		{
  			for ( int y = 0; y < 256; y++ )
  			{
  				long lTotal = 0;

				for ( long lTextureLevel = 0; lTextureLevel < _lCloudTextures; lTextureLevel++ )
				{
					lTotal += (long)( _LargeTextures[ lTextureLevel * _lCloudTextureSize + lX + x + ( lY + y ) * _lCloudTextureDimension ] ) * ( 1 << lTextureLevel );
				}
				lTotal /= ( 1 << ( _lCloudTextures - 1 ) );

  				long lCloudDensity = lTotal - g_lCloudCover;
  				if ( lCloudDensity < 0 )
  					lCloudDensity = 0;
  				if ( lCloudDensity > 255 )
  					lCloudDensity = 255;

  				long lCloudColor = _CloudLookupTable[ lCloudDensity ];
  				
  				_FinalTextures[i][x][y][1] = lCloudColor;
  				_FinalTextures[i][x][y][0] = lCloudColor;
  			}
  		}
	}


	_SkyTextures.resize( _lCloudFinalTextures );

	for ( long lTexture = 0; lTexture < _lCloudFinalTextures; lTexture++ )
	{
		float fBorderColor[] = { 1, 1, 1, 0 };
  		_SkyTextures[ lTexture ].Create();

  		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
  		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR );
  		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE );
  		glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE );
		glTexParameterfv( GL_TEXTURE_2D, GL_TEXTURE_BORDER_COLOR, fBorderColor );
  		glTexEnvf( GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE );
  		glTexImage2D( GL_TEXTURE_2D, 0, GL_LUMINANCE8_ALPHA8, 256, 256, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, _FinalTextures[ lTexture ] );
	}

	_nSkyDisplayList = glGenLists( 1 );

	// Display list for quickly drawing the whole sky
	glNewList( _nSkyDisplayList, GL_COMPILE );
	{
		glDepthMask( false );

		float fSphereRadius = 700;
		float fSphereOffset = 680;
		float fSkySize = 380;
		float fTextureSize = fSkySize / g_lCloudDetailLevel;
		float fSegmentSize = fTextureSize / g_lCloudTextureSegments;
		
		for ( int i = 0; i < _lCloudFinalTextures; i++ )
		{
			_SkyTextures[ i ].Bind();
			// Identify which square of the giant texture we are working on
			float fXOffset = ( i % g_lCloudDetailLevel ) * fTextureSize - fSkySize / 2;
			float fYOffset = ( i / g_lCloudDetailLevel ) * fTextureSize - fSkySize / 2;

			for ( int x = 0; x < g_lCloudTextureSegments; x++ )
			{
				glBegin( GL_TRIANGLE_STRIP );
				{
					for ( int y = 0; y < g_lCloudTextureSegments + 1; y++ )
					{
						float fX = x * fSegmentSize + fXOffset;
						float fY = y * fSegmentSize + fYOffset;
						float fLastX = ( x + 1 ) * fSegmentSize + fXOffset;
						float fZ = sqrt( fSphereRadius * fSphereRadius - ( fX * fX + fY * fY ) ) - fSphereOffset;
						float fLastZ = sqrt( fSphereRadius * fSphereRadius - ( fLastX * fLastX + fY * fY ) ) - fSphereOffset;

						glTexCoord2f( (float)y / (float)g_lCloudTextureSegments, (float)( x + 1 ) / (float)g_lCloudTextureSegments );
						glVertex3f( fLastX, fY, fLastZ );
						glTexCoord2f( (float)y / (float)g_lCloudTextureSegments, (float)x / (float)g_lCloudTextureSegments );
						glVertex3f( fX, fY, fZ );
					}
				}
				glEnd();
			}

			glDepthMask( true );
		}
/*		float fSphereRadius = 200;
		
		float fMaxTexRadius = g_lCloudDetailLevel;

		for ( int lTexture = 0; lTexture < 4; lTexture++ )
		{
			float fLastRadius = 0;
			float fLastZ = fSphereRadius;
			float fLastTexRadius = 0;

  			for ( int zAngle = 1; zAngle < 60; zAngle++ )
  			{
  				float fZ = fSphereRadius * cos( DEGTORAD( zAngle ) ) - 130;
  				float fRadius = fSphereRadius * sin( DEGTORAD( zAngle ) );
//				float fTexRadius = (float)zAngle / 60;
  				// This fakes an infinite plane on the sky dome
				float fTexRadius = (float)zAngle / ( 60 * cos( DEGTORAD( zAngle ) ) );
// 				float fTexRadius = (float)zAngle / 60 * cos( DEGTORAD( zAngle % 90 - 45 ) ) );
  				
  				glBegin( GL_TRIANGLE_STRIP );
  				{
  					for ( int sliceAngle = lTexture * 90; sliceAngle <= lTexture * 90 + 90; sliceAngle += 10 )
  					{
  						float fX, fY;
  						
  						fX = sin( DEGTORAD( sliceAngle ) );
  						fY = cos( DEGTORAD( sliceAngle ) );
  						glTexCoord2f( fLastTexRadius * fX, fLastTexRadius * fY );
  						glVertex3f( fLastRadius * fX, fLastRadius * fY, fLastZ );

  						fX = sin( DEGTORAD( sliceAngle ) );
  						fY = cos( DEGTORAD( sliceAngle ) );
  						glTexCoord2f( fTexRadius * fX, fTexRadius * fY );
  						glVertex3f( fRadius * fX, fRadius * fY, fZ );			
  					}
  				}
  				glEnd();
  				
  				fLastRadius = fRadius;
  				fLastZ = fZ;
  				fLastTexRadius = fTexRadius;
  			}
		}*/
	}
	glEndList();

//	GLenum  glError = glGetError();
}

//===========================================================================================================================

void CClouds::Tick( long lMSElapsed )
{
/*	static long lTimeElapsed = 0;
	lTimeElapsed += lMSElapsed;

	if ( lTimeElapsed > 50 )
	{
		lTimeElapsed = 0;

		GLTeardown();
		GLSetup();
	}*/
}

//===========================================================================================================================

void CClouds::GLTeardown()
{
	glDeleteLists( _nSkyDisplayList, 1 );	

	delete[] _Textures;
	delete[] _LargeTextures;
	delete[] _LargeTextureScratchpad;
	delete[] _FinalTextures;
}

//===========================================================================================================================

void CClouds::GLRender()
{
	glCallList( _nSkyDisplayList );
	return;

/*	glDisable( GL_DEPTH_TEST );
	glEnable( GL_BLEND );
	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	{
	glLoadIdentity();
	//		gluOrtho2D( 0, _lWidth, _lHeight, 0 );
	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	{
	glLoadIdentity();
	*/	
	
/*	float fValues[] = {
		0, 0, -400, -400, 40,
			0, 2, -400,  400, 40,
			2, 2,  400,  400, 40,
			2, 0,  400, -400, 40,
												
			0, 0, 0,   0,   0,
			0, 2, 0,   256, 0,
			2, 2, 256, 256, 0,
			2, 0, 256, 0,   0,
			
			0, 0, 0,   0,   0,
			0, 4, 0,   256, 0,
			4, 4, 256, 256, 0,
			4, 0, 256, 0,   0,
			
			0, 0, 0,   0,   0,
			0, 8, 0,   256, 0,
			8, 8, 256, 256, 0,
			8, 0, 256, 0,   0,
	};
	
	glInterleavedArrays( GL_T2F_V3F, 0, fValues );
	glDisable( GL_CULL_FACE );*/
//	glDisable( GL_TEXTURE_2D );
	
	//	glDisable( GL_TEXTURE_2D );
//	glColor4f( 1, 1, 1, 1 );


	/*	glBegin( GL_QUADS );
	{
	glVertex3f( 1, 1, 1 );
	glVertex3f( 0, 1, 1 );
	glVertex3f( 0, 0, 1 );
	glVertex3f( 1, 0, 1 );
	}
	glEnd();
	
	 glPolygonMode( GL_FRONT, GL_LINE );
	*/
	
	//	GLUquadric* pQuadric = gluNewQuadric();
	//	gluSphere( pQuadric, 1, 10, 10 );
	//	gluDeleteQuadric( pQuadric );
	
	//	glEnable( GL_TEXTURE_GEN_S );
	//	glEnable( GL_TEXTURE_GEN_T );
	
	/*	glEnable( GL_BLEND );
	glDisable( GL_FOG );
	glDepthMask( false );
	
	 glBlendFunc(GL_ONE, GL_ZERO);
	 glColor4f( 1, 1, 1, 1 );
	 glDrawArrays( GL_QUADS, 0, 4 );												
	 glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
	 glDisable( GL_TEXTURE_2D );
	 glColor4f( 0.21f, 0.39f, 0.56f, 0.5 );
	 glDrawArrays( GL_QUADS, 0, 4 );												
	 glEnable( GL_TEXTURE_2D );
	 
	  glDepthMask( true );
	  glEnable( GL_FOG );
	  glDisable( GL_BLEND );
	  
	   glEnable( GL_CULL_FACE );
	*/
	
	//			glColor4f( 1, 1, 1, 0.5 );
	//			glDrawArrays( GL_QUADS, 8, 4 );												
	//		glDrawArrays( GL_QUADS, 4, 4 );
	//	glDrawArrays( GL_QUADS, 0, 4 );
	
	//			glFlush();
	//			unsigned char Pixels[3][256][256];
	
	//			glReadPixels( 0, 0, 256, 256, GL_RGB, GL_UNSIGNED_BYTE, Pixels );
	
	//			for ( int x = 0; x < 256; x++ )
	//			{
	//				for ( int y = 0; y < 256; y++ )
	//				{
	//					Pixels[0][x][y] = 128;
	//				}
	//			}
	
	//			glRasterPos2i( 0, 256 );
	//			glDrawPixels( 256, 256, GL_RGB, GL_UNSIGNED_BYTE, Pixels );
	/*	  }
	glPopMatrix();
	glMatrixMode( GL_PROJECTION );
	}
	glPopMatrix();
	glDisable( GL_BLEND );
	glEnable( GL_DEPTH_TEST );
	glEnable( GL_CULL_FACE );
	*/
	//  glDeleteTextures( 4, nTextures );
}
