/***************************************************************************
                          engine.cpp  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/

#include <stdio.h>
#include <SDL.h>
#include "engine.h"
#include "vector3.h"

#define PI	3.14159265323
#define DEGTORAD( x ) float( x * PI / 180 )
#define RADTODEG( x ) float( x * 180 / PI )

//===========================================================================================================================

CEngine::CEngine()
{
	_lHeight = _lWidth = 1;
	
	_fX = _mMap.GetXSize() / 2.0f * SECTOR_SIZE;
	_fY = _mMap.GetYSize() / 2.0f * SECTOR_SIZE;
	_fvX = _fvY = _fvZ = 0;
	_bOnGround = true;
	
	_fBearing = 45;
	_fInclination = 0;
	
	_bBindings.BindEventVariable( "+showconsole", &g_bShowConsole );
	_bBindings.BindEventVariable( "+forward", &g_bMoveForward );
	_bBindings.BindEventVariable( "+backward", &g_bMoveBackward );
	_bBindings.BindEventVariable( "+jetpack", &g_bJetpack );
	_bBindings.BindEventVariable( "jump", &g_bJump );
	_bBindings.BindEventVariable( "toggle_wireframe", &g_bToggleWireframe );
	
	_bBindings.BindAxisVariable( "x", &_fXAxis );
	_bBindings.BindAxisVariable( "y", &_fYAxis );
	
	_nSkyDisplayList = -1;
}

//===========================================================================================================================

CEngine::~CEngine()
{
}

//===========================================================================================================================

CBindings* CEngine::GetBindings()
{
	return &_bBindings;
}

//===========================================================================================================================

CConsole* CEngine::GetConsole()
{
	return &_cConsole;
}

//===========================================================================================================================

void _SetColor( float z )
{
	static int last_c = -1;
	int c = 0;
	
	if ( z < 0.2 )
		c = 0;
	else if ( z < 2 )
		c = 1;
	else if ( z < 5 )
		c = 2;
	else 
		c = 3;
	
	if ( c == last_c )
		return;
	
	switch ( c )
	{
	case 0:
		glColor3f( 0, 0, 0.7 );
		break;
	case 1:
		glColor3f( 0.5, 0.5, 0 );
		break;
	case 2:
		glColor3f( 0, 0.7, 0 );
		break;
	case 3:
		glColor3f( 0.7, 0.7, 0.7 );
		break;
	}
}

//===========================================================================================================================

void CEngine::GLSetup()
{
	g_bShowConsole = g_bMoveForward = g_bMoveBackward = false;
	_fXAxis = _fYAxis = 0.0f;

	GLint iViewport[ 4 ];
	
	glEnable( GL_TEXTURE_2D );
	
//	_tSBFront.Load( "sbfront" );
//	_tSBBack.Load( "sbback" );
//	_tSBLeft.Load( "sbleft" );
//	_tSBRight.Load( "sbright" );
//	_tSBUp.Load( "sbup" );

	_mMap.GLSetup();
	_cClouds.GLSetup();
	_cConsole.GLSetup();

	glGetIntegerv( GL_VIEWPORT, iViewport );
	GLResize( iViewport[ 2 ], iViewport[ 3 ] );
	
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	
	glClearDepth( 1.0f );
	glEnable( GL_DEPTH_TEST );
	
	glColorMaterial( GL_FRONT, GL_AMBIENT_AND_DIFFUSE );
	glEnable( GL_COLOR_MATERIAL );
	
//	glDisable( GL_LIGHTING );
//	glEnable( GL_LIGHTING );
//2	glEnable( GL_LIGHT0 );
	
	glClearColor( 0.21f, 0.39f, 0.56f, 1.0f );
	GLfloat fogColor[] = {0.21,0.39,0.56,1};   // fog color
	
	// execute OpenGL commands
	
	glFogi (GL_FOG_MODE, GL_LINEAR);   // Fog fade using exponential function
	glFogfv (GL_FOG_COLOR, fogColor);   // Set the fog color
	glFogf (GL_FOG_DENSITY, 0.35);   // Set the density, don't make it too high.
	glHint (GL_FOG_HINT, GL_NICEST);   // ROS says Set default calculation mode
	//	glFogf( GL_FOG_START, 130.0f );
	glFogf( GL_FOG_START, 100.0f );
	glFogf( GL_FOG_END, 150.0f );
	glEnable ( GL_FOG );   // turn on fog, otherwise you won't see any
	
	//	glPolygonMode( GL_FRONT, GL_LINE );
	glCullFace( GL_BACK );
	glEnable( GL_CULL_FACE );
	
	//	glEnable( GL_LIGHTING );
	glEnable( GL_LIGHT0 );
	
	//	double fX, fY, fZ;
	//	fX = _fDistance * sin( DEGTORAD( _fBearing ) ) * cos( DEGTORAD( _fInclination ) ) + _vCenter[ 0 ];
	//	fY = _fDistance * cos( DEGTORAD( _fBearing ) ) * cos( DEGTORAD( _fInclination ) ) + _vCenter[ 1 ];
	//	fZ = _fDistance * sin( DEGTORAD( _fInclination ) ) + _vCenter[ 2 ];
	//	gluLookAt( XSIZE * 1.5, YSIZE * 1.5, ( XSIZE + YSIZE ) * 0.75, 0, 0, 0, 0, 0, 1 );
	//	gluLookAt( 0, 0, 4, 0, 0, 8, 0, 0, 1 );
	
	_nSkyDisplayList = glGenLists( 1 );
	
/*	glNewList( _nSkyDisplayList, GL_COMPILE );
	{
		glDepthMask( false );
		
		float fValues[100] = {
			// front
			0, 0, -1, +1, +1,
				0, 1, -1, +1, -1,
				1, 1, +1, +1, -1,
				1, 0, +1, +1, +1,
				
				// right
				0, 0, +1, +1, +1,
				0, 1, +1, +1, -1,
				1, 1, +1, -1, -1,
				1, 0, +1, -1, +1,
		          				
				// back
				0, 0, +1, -1, +1,
				0, 1, +1, -1, -1,
				1, 1, -1, -1, -1,
				1, 0, -1, -1, +1,
		          				
				// left
				0, 0, -1, -1, +1,
				0, 1, -1, -1, -1,
				1, 1, -1, +1, -1,
				1, 0, -1, +1, +1,
				
				// up
				0, 0, +1, -1, +1,
				0, 1, -1, -1, +1,
				1, 1, -1, +1, +1,
				1, 0, +1, +1, +1,          					
		};         								
		
		glInterleavedArrays( GL_T2F_V3F, 0, fValues );		          				
		
		_tSBFront.Bind();
		glDrawArrays( GL_QUADS, 0, 4 );
		_tSBRight.Bind();
		glDrawArrays( GL_QUADS, 4, 4 );
		_tSBBack.Bind();
		glDrawArrays( GL_QUADS, 8, 4 );
		_tSBLeft.Bind();
		glDrawArrays( GL_QUADS, 12, 4 );
		_tSBUp.Bind();
		glDrawArrays( GL_QUADS, 16, 4 );
		
		glDepthMask( true );
	}
	glEndList();*/
}

//===========================================================================================================================

void CEngine::GLTeardown()
{
	glDeleteLists( _nSkyDisplayList, 1 );

	_mMap.GLTeardown();
	_cConsole.GLTeardown();
	_cClouds.GLTeardown();
}

//===========================================================================================================================

void CEngine::Setup()
{
	_mMixer.Setup( 8 );

	_mMixer.Load( "footstep-left", &_sLeft );
	_mMixer.Load( "footstep-right", &_sRight );
	_mMixer.Load( "jetpack", &_sJetpack );
}

//===========================================================================================================================

void CEngine::Teardown()
{
	_mMixer.Teardown();
}

//===========================================================================================================================

void CEngine::Tick( long lMSElapsed )
{
	_fBearing += _fXAxis * 2.0f;
	_fInclination += _fYAxis * 2.0f;
	
	if ( _fBearing > 360 )
		_fBearing -= 360;
	if ( _fBearing < 0 )
		_fBearing += 360;

	if ( _fInclination > 90 )
		_fInclination = 90;
	if ( _fInclination < -90 )
		_fInclination = -90;

	if ( g_bToggleWireframe )
	{
		g_bToggleWireframe = false;
		_bRenderWireframe = !_bRenderWireframe;
	}

//	float fDistance = 0.005 * lMSElapsed;
//	float fDistance = 0.005 * lMSElapsed;

/*	if ( g_bMoveForward )
	{
		_fX += fDistance * sin( DEGTORAD( _fBearing ) ) * cos( DEGTORAD( _fInclination ) );
		_fY += fDistance * cos( DEGTORAD( _fBearing ) ) * cos( DEGTORAD( _fInclination ) );
		_fZ += fDistance * sin( DEGTORAD( _fInclination ) );
	}
	
	if ( g_bMoveBackward )
	{
		_fX -= fDistance * sin( DEGTORAD( _fBearing ) ) * cos( DEGTORAD( _fInclination ) );
		_fY -= fDistance * cos( DEGTORAD( _fBearing ) ) * cos( DEGTORAD( _fInclination ) );
		_fZ -= fDistance * sin( DEGTORAD( _fInclination ) );
	}*/

	float fNewSpeedX = _fvX, fNewSpeedY = _fvY, fNewSpeedZ = _fvZ;
	float fMaxSpeed = 4;
    bool bAdjustMaxSpeed = false;

	float fStrideDistance = 3.5;
	static float fLeftStride = 0, fRightStride = 0;

	bool bWalking = false;
	bool bFlying = false;

	if ( _bOnGround )
	{
		const float fWalkVelocity = 4;
	
    	if ( g_bMoveForward )
    	{
    		fNewSpeedX = _fvX + fWalkVelocity * sin( DEGTORAD( _fBearing ) );
    		fNewSpeedY = _fvY + fWalkVelocity * cos( DEGTORAD( _fBearing ) );
			bAdjustMaxSpeed = true;
			bWalking = true;
    	}
    	
    	if ( g_bMoveBackward )
    	{
    		fNewSpeedX = _fvX - fWalkVelocity * sin( DEGTORAD( _fBearing ) );
    		fNewSpeedY = _fvY - fWalkVelocity * cos( DEGTORAD( _fBearing ) );
			bAdjustMaxSpeed = true;
			bWalking = true;
    	}
	}

	if ( bWalking )
	{
		fLeftStride += 4 * (float)lMSElapsed / 1000.0f;
		fRightStride += 4 * (float)lMSElapsed / 1000.0f;

		if ( fLeftStride > fStrideDistance )
		{
			_mMixer.Play( _sLeft );
			fLeftStride -= fStrideDistance;
		}

		if ( fRightStride > fStrideDistance )
		{
			_mMixer.Play( _sRight );
			fRightStride -= fStrideDistance;
		}
	}
	else
	{
		fLeftStride = fStrideDistance * 0.25;
		fRightStride = fStrideDistance * 0.75;
	}

	if ( g_bJetpack )
	{
		_mMixer.PlayLooped( _sJetpack );

		fMaxSpeed = 10;
		_bOnGround = false;
		bAdjustMaxSpeed = true;
		
		float fJetpackAcceleration = 30 * (float)lMSElapsed / 1000.0f;
		
		if ( g_bMoveForward )
		{
       		fNewSpeedX = _fvX + fJetpackAcceleration * sin( DEGTORAD( _fBearing ) );
       		fNewSpeedY = _fvY + fJetpackAcceleration * cos( DEGTORAD( _fBearing ) );
			fNewSpeedZ = _fvZ + fJetpackAcceleration / 4;
		}
		else
		if ( g_bMoveBackward )
		{
       		fNewSpeedX = _fvX - fJetpackAcceleration * sin( DEGTORAD( _fBearing ) );
       		fNewSpeedY = _fvY - fJetpackAcceleration * cos( DEGTORAD( _fBearing ) );
			fNewSpeedZ = _fvZ + fJetpackAcceleration / 4;
		}
		else
		{
			fNewSpeedX = _fvX;
			fNewSpeedY = _fvY;
			fNewSpeedZ = _fvZ + fJetpackAcceleration / 2;
		}
		
	}
	else
	{
		_mMixer.Stop( _sJetpack );
	}

	if ( bAdjustMaxSpeed )
	{
       	float fNewSpeed = fNewSpeedX * fNewSpeedX + fNewSpeedY * fNewSpeedY;
    	_fvX = fNewSpeedX;
    	_fvY = fNewSpeedY;
       		
       	if ( fNewSpeed > fMaxSpeed * fMaxSpeed )
       	{
       		float fDirectionalSpeed = fMaxSpeed / sqrt( fNewSpeed );
       		_fvX *= fDirectionalSpeed;
       		_fvY *= fDirectionalSpeed;
       	}
    		
       	if ( fNewSpeedZ < fMaxSpeed )
     		_fvZ = fNewSpeedZ;
	} 			
	
	if ( g_bJump )
	{
		if ( _bOnGround )
		{
			_fvZ += 5;
			_bOnGround = false;			
		}
		
		g_bJump = false;
	}
	
   	_fX += _fvX * (float)lMSElapsed / 1000.0f;
   	_fY += _fvY * (float)lMSElapsed / 1000.0f;
   	_fZ += _fvZ * (float)lMSElapsed / 1000.0f;
		
	float fZHeight = _mMap.GetHeight( _fX, _fY ) + 0.5;
	
   	_fvZ -= 9.81f * (float)lMSElapsed / 1000.0f;
		
   	if ( _fZ <= fZHeight )
   	{
   		if ( !_bOnGround )			
   		{
    		_cConsole.AddMessage( "Landed! (Z velocity = %f m/s)", _fvZ );

    		if ( !g_bJetpack )
    		{
    			_bOnGround = true;
    			_fvZ = 0;
    		}
    		else
    		{
    			_fvZ = -_fvZ / 10;
    		}
		}
		else
		{
			// Calculate ground friction
			_fvX *= 0.7;
			_fvY *= 0.7;
			
			_fvZ = 0;
		}
		
   		_fZ = fZHeight;
  	}

   	if ( _bOnGround )
   	{
   		_fZ = fZHeight;
   	}
   		
	_cClouds.Tick( lMSElapsed );
}

//===========================================================================================================================

void CEngine::GLResize( long lWidth, long lHeight )
{
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	
	_lWidth = lWidth;
	_lHeight = lHeight;
	
	glViewport( 0, 0, _lWidth, _lHeight );
	
	GLdouble dAspect, dNearPlane, dFarPlane;
	
	dNearPlane = 0.5;
	dFarPlane = 20000.0;
	if ( _lHeight )
		dAspect = ( double )_lWidth / ( double )_lHeight;
	else    // don't divide by zero, not that we should ever run into that...
		dAspect = 1.0;
	gluPerspective( 30.0, dAspect, dNearPlane, dFarPlane );
	
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	gluLookAt( 0, 0, 0, 0, 1, 0, 0, 0, 1 );
}

//===========================================================================================================================

void CEngine::GLRender() 
{
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
	
/*	glDisable( GL_TEXTURE_2D );
	
	glColor3f( 0, 0, 0 );*/
	glEnable( GL_TEXTURE_2D );

	glMatrixMode( GL_MODELVIEW );
	glPushMatrix();
	{
		glRotated( -_fInclination, 1.0f, 0.0f, 0.0f );
		glRotated( _fBearing, 0.0f, 0.0f, 1.0f );
		
/*		float fLight[] = { 10, 10, 3, 0.0 };
		float fLightAmbientColor[] = { 1, 1, 1, 1 };
		float fLightDiffuseColor[] = { 1, 1, 1, 1 };
		//	glLightfv( GL_LIGHT0, GL_AMBIENT, fLightAmbientColor );
		glLightfv( GL_LIGHT0, GL_DIFFUSE, fLightDiffuseColor );
		glLightfv( GL_LIGHT0, GL_POSITION, fLight );
		
*/		//		glCallList( _nSkyDisplayList );
		
		_cClouds.GLRender();
		
		glTranslated( -_fX, -_fY, -_fZ );
		//		glEnable( GL_LIGHTING );
		
		//		glCallList( _nDisplayList );
		
		
//		long lLastTexture = -1;
		
//		glEnable( GL_LIGHTING );

		if ( _bRenderWireframe )
		{
			glPolygonMode( GL_FRONT, GL_LINE );
			glDisable( GL_TEXTURE_2D );
			glDisable( GL_FOG );
		}

		_mMap.GLRender( _fX, _fY, _fZ );

		if ( _bRenderWireframe )
		{
			glPolygonMode( GL_FRONT, GL_FILL );
			glEnable( GL_TEXTURE_2D );
			glEnable( GL_FOG );
		}

/*		glColor3f( 1.0, 1.0, 1.0 );
		long lX = _mMap.GetXPosition( _fX );
		long lY = _mMap.GetXPosition( _fY );
		for ( long x = -30; x <= +30; x++ )
		{
			for ( long y = -30; y <= +30; y++ )
			{
				long lSectorX = lX + x;
				long lSectorY = lY + y;
				if ( lSectorX < 0 || lSectorY < 0 || lSectorX >= _mMap.GetXSize() || lSectorY >= _mMap.GetYSize() )
					continue;
				
				if ( x * x + y * y > 60 * 60 )
					continue;

				float fHeight = _mMap.GetMapHeight( lSectorX, lSectorY );

				if ( fHeight > 7 )
				{
					if ( lLastTexture != _tRock.ID() )
					{
						_tRock.Bind();
						lLastTexture = _tRock.ID();
					}
				}
				else if ( fHeight > 0.1 )
				{
					if ( lLastTexture != _tGrass.ID() )
					{
						_tGrass.Bind();
						lLastTexture = _tGrass.ID();
					}
				}
				else
				{
					if ( lLastTexture != _tWater.ID() )
					{
						_tWater.Bind();
						lLastTexture = _tWater.ID();
					}
				}
				
				vector3 vEye( _fX, _fY, _fZ );
				_mMap.DrawSector( vEye, lSectorX, lSectorY );
			}
		}

		glEnable( GL_TEXTURE_2D );
//		glDisable( GL_LIGHTING );
		/*		GLUquadricObj* pQuadric;
		pQuadric = gluNewQuadric();
		glTranslated( 20, 20, 1 );
		gluSphere( pQuadric, 2, 10, 10 );
		gluDeleteQuadric( pQuadric );
		*/
		//		glDisable( GL_LIGHTING );
	}
	glPopMatrix();
	
	if ( g_bShowConsole )
		_cConsole.GLRender();

	glFinish();
	
	//	GLenum glError = glGetError();
}

