/***************************************************************************
                          engine.h  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/


#ifndef ENGINE_H
#define ENGINE_H

#include "glplatform.h"
#include "map.h"
#include "texture.h"
#include "bindings.h"
#include "clouds.h"
#include "console.h"
#include "mixer.h"
#include "sound.h"

class CEngine
{
public:
	CEngine();
	virtual ~CEngine();

	void GLSetup();
	void GLTeardown();
	void GLResize( long lWidth, long lHeight );
	void GLRender();

	void Tick( long lMSElapsed );
	void Setup();
	void Teardown();

	float GetFPS();

	CBindings* GetBindings();
	CConsole* GetConsole();
protected:
	float _GetHeight( float fX, float fY );

	// Bound one-shot events

	// Bound parameter one-shot events

	// Bound on/off events

	// Bound event variables
	bool g_bMoveForward;
	bool g_bMoveBackward;
	bool g_bJump;
	bool g_bJetpack;
	bool g_bShowConsole;
	bool g_bToggleWireframe;
	
	float _fXAxis;
	float _fYAxis;

	// Bound variables
	
	long _lHeight, _lWidth;

	float	_fBearing, _fInclination;
	float	_fX, _fY, _fZ;
	float	_fvX, _fvY, _fvZ;

	bool	_bOnGround;
	bool	_bRenderWireframe;

	int	_nSkyDisplayList;

	CMap		_mMap;

	CClouds		_cClouds;

	CMixer		_mMixer;

	CBindings	_bBindings;
	CConsole	_cConsole;

	CSound		_sLeft;
	CSound		_sRight;
	CSound		_sJetpack;
};

#endif
