/***************************************************************************
                          main.cpp  -  description
                             -------------------
    begin                : Sat Sep 30 23:04:17 MDT 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/


#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#define _SDL_main_h
#include <SDL.h>

#include <string>

#include "engine.h"
#include "console.h"

#define XRES 800
#define YRES 600

CEngine g_e;
CConsole* g_pc;

/*void g_pc->AddMessage( const char* szMessage, ... )
{
	const long MAX_LENGTH = 512;
	char szBuff[ MAX_LENGTH ];

	va_list args;

	va_start( args, szMessage );

#ifdef _WIN32
	_vsnprintf( szBuff, MAX_LENGTH, szMessage, args );
#else
	vsnprintf( szBuff, MAX_LENGTH, szMessage, args );
#endif
	szBuff[ MAX_LENGTH ] = 0;

	va_end( args );

#ifdef _WIN32
	OutputDebugString( szBuff );
	OutputDebugString( "\n" );
#else
	cout << szBuff << endl;
#endif

	g_LastMessages.push_back( szBuff );
	while ( g_LastMessages.size() > 40 )
	{
		g_LastMessages.pop_front();
	}
}
*/
SDL_Window* _InitializeVideo( bool bFullscreen )
{
	/* Initialize SDL for video output */
	if ( SDL_Init(SDL_INIT_VIDEO) < 0 ) {
		fprintf(stderr, "Unable to initialize SDL: %s\n", SDL_GetError());
		exit(1);
	}

	Uint32 uiFlags = SDL_WINDOW_OPENGL;
	if ( bFullscreen )
		uiFlags |= SDL_WINDOW_FULLSCREEN;
	SDL_GL_SetAttribute( SDL_GL_RED_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_GREEN_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_BLUE_SIZE, 5 );
	SDL_GL_SetAttribute( SDL_GL_DEPTH_SIZE, 32 );
	SDL_GL_SetAttribute( SDL_GL_DOUBLEBUFFER, 1 );
	// long lBPP = SDL_VideoModeOK( XRES, YRES, 16, uiFlags );

	// g_pc->AddMessage( "Picked resolution of %ix%ix%i", XRES, YRES, lBPP );
	SDL_Window *screen = SDL_CreateWindow("GL Landscape",
                          SDL_WINDOWPOS_UNDEFINED,
                          SDL_WINDOWPOS_UNDEFINED,
                          XRES, YRES,
                          uiFlags);

	SDL_GLContext gl = SDL_GL_CreateContext(screen);
	/* Create a 640x480 OpenGL screen */
	// if ( SDL_SetVideoMode( XRES, YRES, lBPP, uiFlags ) == NULL ) {
	// 	fprintf(stderr, "Unable to create OpenGL screen: %s\n", SDL_GetError());
	// 	SDL_Quit();
	// 	exit(2);
	// }
	
	g_pc->AddMessage( "GL_VENDOR: %s", glGetString( GL_VENDOR ) );
	g_pc->AddMessage( "GL_RENDERER: %s", glGetString( GL_RENDERER ) );
	g_pc->AddMessage( "GL_VERSION: %s", glGetString( GL_VERSION ) );
	g_pc->AddMessage( "GL_EXTENSIONS: %s", glGetString( GL_EXTENSIONS ) );
	
	SDL_SetRelativeMouseMode(SDL_TRUE);
	
	g_e.GLSetup();
	return screen;
}

void _ShutdownVideo()
{
	g_e.GLTeardown();

	SDL_ShowCursor( 1 );

	SDL_Quit();
}

void _UpdateFPS()
{
	static long dwLastTime = -1;
	static long dwCount = 0;
	
	dwCount++;
	
	if ( dwLastTime != -1 )
	{
		long dwCurrentTime = SDL_GetTicks();
		
		if ( dwCurrentTime - dwLastTime > 1000 )
		{
			long dwElapsed = dwCurrentTime - dwLastTime;
			float fFPS = (float)dwCount * 1000.0 / (float)dwElapsed;
			
			dwCount = 0;
			dwLastTime = dwCurrentTime;

			char szText[ 500 ];
			sprintf( szText, "GL Landscape: %0.2f fps", fFPS );

			// SDL_WM_SetCaption( szText, NULL );
		}
	}
	else
		dwLastTime = SDL_GetTicks();
}

int main(int argc, char **argv)
{
	bool bDone;
	bool bCaptureMouse = true;
	bool bFullscreen = true;
	SDL_Window *window;

	g_e.Setup();

	g_pc = g_e.GetConsole();

 	window = _InitializeVideo( bFullscreen );

	// SDL_WarpMouse( 200, 200 );

	bDone = false;
	while ( !bDone ) 
	{
		_UpdateFPS();
		
		if ( bCaptureMouse )
		{
			int nX, nY;
			SDL_GetRelativeMouseState( &nX, &nY );
			// SDL_WarpMouse( 200, 200 );
			
			*( g_e.GetBindings()->GetAxisVariable( "x" ) ) = float( nX ) / 50.0;
			*( g_e.GetBindings()->GetAxisVariable( "y" ) ) = float( nY ) / 50.0;
		}
		else
		{
			*( g_e.GetBindings()->GetAxisVariable( "x" ) ) = 0;
			*( g_e.GetBindings()->GetAxisVariable( "y" ) ) = 0;
		}

		static unsigned long ulTime = SDL_GetTicks();
		unsigned long ulCurrentTime = SDL_GetTicks();

		g_e.Tick( ulCurrentTime - ulTime );
			
		ulTime = ulCurrentTime;
		g_e.GLRender();
		SDL_GL_SwapWindow(window);
		
		/* This could go in a separate function */
		SDL_Event e;
		while ( SDL_PollEvent( &e ) ) 
		{
			switch ( e.type )
			{
			case SDL_QUIT:
				{
					g_pc->AddMessage( "SDL_QUIT" );
					bDone = true;	
				}
				break;

			case SDL_KEYDOWN:
				{
					g_pc->AddMessage( "SDL_KEYDOWN: sym = %i", e.key.keysym.sym );

					switch ( e.key.keysym.sym )
					{
					case SDLK_ESCAPE:
						bDone = true;
						break;

					case SDLK_m:
						bCaptureMouse = !bCaptureMouse;
						break;

					case SDLK_f:
						bFullscreen = !bFullscreen;
						_ShutdownVideo();
						_InitializeVideo( bFullscreen );
						break;
						
					case SDLK_w:
						*( g_e.GetBindings()->GetEventVariable( "toggle_wireframe" ) ) = true;
						break;

					case SDLK_SPACE:
   						*( g_e.GetBindings()->GetEventVariable( "jump" ) ) = true;
						break;
					
					case SDLK_LSHIFT:
   						*( g_e.GetBindings()->GetEventVariable( "+jetpack" ) ) = true;
						break;
						
					default:
						;
					}
				}
				break;

			case SDL_KEYUP:
				{
					g_pc->AddMessage( "SDL_KEYUP: sym = %i", e.key.keysym.sym );

					switch ( e.key.keysym.sym )
					{
					case SDLK_LSHIFT:
   						*( g_e.GetBindings()->GetEventVariable( "+jetpack" ) ) = false;
						break;
						
					default:
						;
					}
				}
				break;
				
			case SDL_MOUSEBUTTONDOWN:
				{
					g_pc->AddMessage( "SDL_MOUSEBUTTONDOWN: button = %i", e.button.button );

					if ( e.button.button == SDL_BUTTON_LEFT )
						*( g_e.GetBindings()->GetEventVariable( "+showconsole" ) ) = true;
					if ( e.button.button == SDL_BUTTON_MIDDLE )
						*( g_e.GetBindings()->GetEventVariable( "+backward" ) ) = true;
					if ( e.button.button == SDL_BUTTON_RIGHT )
						*( g_e.GetBindings()->GetEventVariable( "+forward" ) ) = true;
				}
				break;

			case SDL_MOUSEBUTTONUP:
				{
					g_pc->AddMessage( "SDL_MOUSEBUTTONUP: button = %i", e.button.button );

					if ( e.button.button == SDL_BUTTON_LEFT )
						*( g_e.GetBindings()->GetEventVariable( "+showconsole" ) ) = false;
					if ( e.button.button == SDL_BUTTON_MIDDLE )
						*( g_e.GetBindings()->GetEventVariable( "+backward" ) ) = false;
					if ( e.button.button == SDL_BUTTON_RIGHT )
						*( g_e.GetBindings()->GetEventVariable( "+forward" ) ) = false;
				}
				break;

			default:
				;
			}
		}
	}
	
	_ShutdownVideo();
	
	g_e.Teardown();

	return 1;
}

#ifdef _WIN32

extern "C" int SDL_RegisterApp( char *name, Uint32 style, void *hInst );

int APIENTRY WinMain(HINSTANCE hInstance,
                     HINSTANCE hPrevInstance,
                     LPSTR     lpCmdLine,
                     int       nCmdShow)
{
	SDL_RegisterApp( "GLLandscape", CS_BYTEALIGNCLIENT, ::GetModuleHandle( NULL ) );
	return main( 0, NULL );
}

#endif

