/***************************************************************************
                          map.h  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/


#ifndef MAP_H
#define MAP_H

#include "glplatform.h"
#include "vector3.h"
#include "texture.h"

#define SECTOR_SIZE (4.0f)

class CMap
{
public:
	CMap();
	virtual ~CMap();

	long GetXSize() { return 256; };
	long GetYSize() { return 256; };

	long GetXPosition( float fX ) { return long( fX / SECTOR_SIZE ); }
	long GetYPosition( float fY ) { return long( fY / SECTOR_SIZE ); }

	void GLSetup();
	void GLTeardown();
	void GLRender( float fX, float fY, float fZ );

	void DrawSector( vector3 vEye, long lX, long lY );
	float GetHeight( float fX, float fY );
	float GetInclination( float fX, float fY );
	float GetMapHeight( long lX, long lY );
protected:
	void _Diamond( long lX, long lY, long lSide, float fScale );
	void _Square( long lX, long lY, long lSide, float fScale );
	float _Random();
	float _InterpolateHeight( float fX, float fY, const vector3& v1, const vector3& v2, const vector3& v3 );
	float _GetInclination( const vector3& v1, const vector3& v2, const vector3& v3 );
	void _DrawSquare( float fX, float fY, long lV0X, long lV0Y, long lV1X, long lV1Y, long lV2X, long lV2Y, long lV3X, long lV3Y, long lLOD );
	void _DrawTriangles( float fX, float fY, long lV0X, long lV0Y, long lV1X, long lV1Y, long lV2X, long lV2Y, long lV3X, long lV3Y, long lLOD );
	
	CTexture	_tRock;
	CTexture	_tGrass;
	CTexture	_tWater;

//	float _fHeights[XSIZE][YSIZE];
	float _fLandscape[257][257];
	
	float*	_pfVertices;
	long*	_plQualityIndices;
	long	_lMapSize;

	long	_lDistanceMap[65][65];
	long	_lDetailMap[64][64];
	long	_lLODDistances[6];
};

#endif
