/***************************************************************************
                          mixer.cpp  -  description
                             -------------------
    begin                : Thu Oct 12 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/

#include "mixer.h"
#include "sound.h"

#include <string>
using namespace std;

#ifdef _WIN32
	#pragma comment(lib, "QMDX.lib")
#endif

void CMixer::Setup( long lChannels )
{
#ifdef _WIN32
    QMIXCONFIG mc;
    memset( &mc, 0, sizeof( mc) );
    
	mc.dwSize = sizeof( mc );
    mc.iChannels = lChannels;

	_hQMixer = QMDX_InitEx( &mc );

	QMDX_Activate( _hQMixer, TRUE );
    QMDX_OpenChannel( _hQMixer, lChannels, QMIX_OPENCOUNT );
	QMDX_SetVolume( _hQMixer, 0, QMIX_ALL, 32767 );
#endif
}

void CMixer::Teardown()
{
#ifdef _WIN32
	QMDX_CloseSession( _hQMixer );
#endif
}

long CMixer::GetFreeChannel() const
{
#ifdef _WIN32
	// Find the least-recently used channel
	long lChannel = QMDX_FindChannel( _hQMixer, QMIX_FINDCHANNEL_LRU, 0 );

	return lChannel;
#endif
}

void CMixer::Load( const char* szSound, CSound* pSound )
{
	string strFilename = szSound;
	strFilename = "data/" + strFilename + ".wav";

#ifdef _WIN32
	pSound->_pWave = QMDX_OpenWave( _hQMixer, const_cast< char* >( strFilename.c_str() ), 0, WMIX_FILE );
#endif
}

void CMixer::Play( CSound& Sound )
{
#ifdef _WIN32
	Sound._lChannel = GetFreeChannel();
	QMDX_PlayEx( _hQMixer, Sound._lChannel, QMIX_CLEARQUEUE, Sound._pWave, 0, 0 );
#endif
}

void CMixer::PlayLooped( CSound& Sound )
{
#ifdef _WIN32
	if ( !Sound._bPlaying )
	{
		Sound._lChannel = GetFreeChannel();
		Sound._bPlaying = true;
		QMDX_PlayEx( _hQMixer, Sound._lChannel, QMIX_CLEARQUEUE, Sound._pWave, -1, 0 );
	}
#endif
}

void CMixer::Stop( CSound& Sound )
{
#ifdef _WIN32
	if ( Sound._bPlaying )
	{
		Sound._bPlaying = false;
		QMDX_SetVolume( _hQMixer, Sound._lChannel, QMIX_USEONCE, 0 );
		QMDX_StopChannel( _hQMixer, Sound._lChannel, QMIX_STOP_FINISHLASTLOOP );
	}
#endif
}