/***************************************************************************
                          texture.cpp  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/

#include <SDL.h>
#include <string>

#include "global.h"
#include "texture.h"

//===========================================================================================================================

CTexture::CTexture()
{
	_uiTexture = (GLuint)-1;
}

//===========================================================================================================================

CTexture::~CTexture()
{
	Release();
}

//===========================================================================================================================

void CTexture::Create()
{
	glGenTextures( 1, &_uiTexture );

	glBindTexture( GL_TEXTURE_2D, _uiTexture );
}

//===========================================================================================================================

void CTexture::Load( const char* szTexture )
{
	std::string strFilename = szTexture;
	strFilename = "data/" + strFilename + ".bmp";
	
	Create();
	
	SDL_Surface* pSurface = SDL_LoadBMP( strFilename.c_str() );
	if ( pSurface == NULL )
		return;

	for ( int i = 0; i < pSurface->w * pSurface->h; i++ )
	{
		char c;
		char* pc = ( (char*)pSurface->pixels ) + i * 3;

		c = pc[ 0 ];
		pc[ 0 ] = pc[ 2 ];
		pc[ 2 ] = c;
	}
	
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR_MIPMAP_NEAREST );
//	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP );
//	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP );
	gluBuild2DMipmaps( GL_TEXTURE_2D, GL_RGB8, pSurface->w, pSurface->h, GL_RGB, GL_UNSIGNED_BYTE, pSurface->pixels );

	SDL_FreeSurface( pSurface );
}

//===========================================================================================================================

void CTexture::Bind()
{
	if ( _uiTexture != (GLuint)-1 )
	{
		glBindTexture( GL_TEXTURE_2D, _uiTexture );
	}
}

//===========================================================================================================================

void CTexture::Release()
{
	if ( _uiTexture != (GLuint)-1 )
	{
		glDeleteTextures( 1, &_uiTexture );
		_uiTexture = (GLuint)-1;
	}
}

//===========================================================================================================================

long CTexture::ID()
{
	return (long)_uiTexture;
}
