/***************************************************************************
                          vector3.h  -  description
                             -------------------
    begin                : Sat Sep 30 2000
    copyright            : (c) 2000 by Matthew Mastracci
    email                : mmastrac@canada.com
 ***************************************************************************/


#ifndef VECTOR3_H
#define VECTOR3_H

#include <assert.h>
#include <math.h>
#include <string.h>

/////////////////////////////////////////////////////////////////////////////
// vector3
class vector3
{
public:
	inline vector3() 
		{ memset( &_fValues, 0, sizeof( _fValues ) ); };

	inline vector3( float fX, float fY, float fZ ) 
		{ _fValues[0] = fX; _fValues[1] = fY; _fValues[2] = fZ; };

	inline vector3( const vector3& v ) 
		{ memcpy( &_fValues, &v._fValues, sizeof( _fValues ) ); };

	inline vector3( const float* pfValues ) 
		{ memcpy( &_fValues, pfValues, sizeof( _fValues ) ); };
	
	inline float operator *( const vector3& v ) const
	{
		return _fValues[0] * v._fValues[0] + _fValues[1] * v._fValues[1] + _fValues[2] * v._fValues[2];
	}
		
	inline vector3 operator ^( const vector3& v ) const
	{
		return vector3( _fValues[1] * v._fValues[2] - _fValues[2] * v._fValues[1],
						_fValues[2] * v._fValues[0] - _fValues[0] * v._fValues[2],
						_fValues[0] * v._fValues[1] - _fValues[1] * v._fValues[0] );
	}

	inline vector3 operator +( const vector3& v ) const
	{
		return vector3( _fValues[0] + v._fValues[0], _fValues[1] + v._fValues[1], _fValues[2] + v._fValues[2] );
	}

	inline vector3 operator -( const vector3& v ) const
	{
		return vector3( _fValues[0] - v._fValues[0], _fValues[1] - v._fValues[1], _fValues[2] - v._fValues[2] );
	}

	inline vector3 operator /( float f ) const
	{
		if ( f == 0.0f )
		{
			assert( false );
			return vector3();
		}

		return vector3( _fValues[0] / f, _fValues[1] / f, _fValues[2] / f);
	}

	inline float length() const
	{
		return (float)sqrt( _fValues[0] * _fValues[0] + _fValues[1] * _fValues[1] + _fValues[2] * _fValues[2] );
	}

	inline vector3 normalize() const
	{
		return *this / length();
	}

	inline operator float*() 
		{ return _fValues; };

	inline operator const float*() const
		{ return _fValues; };

	inline float operator[]( int nIndex ) const
		{ return _fValues[ nIndex ]; };

	inline float& operator[]( int nIndex ) 
		{ return _fValues[ nIndex ]; };

protected:
	float _fValues[3];
};

#endif

